@@ -30,6 +30,7 @@
 
 using namespace swoole;
 using swoole::coroutine::Socket;
+using swoole::http2::get_default_setting;
 
 namespace Http2 = swoole::http2;
 
@@ -83,6 +84,10 @@ class Client {
     Http2::Settings local_settings = {};
     Http2::Settings remote_settings = {};
 
+    // flow control
+    uint32_t remote_window_size = 0;
+    uint32_t local_window_size = 0;
+
     std::unordered_map<uint32_t, Stream *> streams;
     std::queue<zend_string *> send_queue;
 
@@ -97,6 +102,7 @@ class Client {
         _zobject = *__zobject;
         zobject = &_zobject;
         Http2::init_settings(&local_settings);
+        local_window_size = local_settings.init_window_size;
     }
 
     inline Stream *get_stream(uint32_t stream_id) {
@@ -468,6 +474,7 @@ bool Client::connect() {
     stream_id = 1;
     // [init]: we must set default value, server is not always send all the settings
     Http2::init_settings(&remote_settings);
+    remote_window_size = remote_settings.init_window_size;
 
     int ret = nghttp2_hd_inflate_new2(&inflater, php_nghttp2_mem());
     if (ret != 0) {
@@ -571,7 +578,7 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: max_concurrent_streams=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_INIT_WINDOW_SIZE:
-                remote_settings.window_size = value;
+                remote_window_size = remote_settings.init_window_size = value;
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: init_send_window=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_MAX_FRAME_SIZE:
@@ -581,14 +588,13 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
             case SW_HTTP2_SETTINGS_MAX_HEADER_LIST_SIZE:
                 if (value != remote_settings.max_header_list_size) {
                     remote_settings.max_header_list_size = value;
-                    /*
+#if 0
                     int ret = nghttp2_hd_inflate_change_table_size(inflater, value);
-                    if (ret != 0)
-                    {
+                    if (ret != 0) {
                         nghttp2_error(ret, "nghttp2_hd_inflate_change_table_size() failed");
                         return SW_ERROR;
                     }
-                    */
+#endif
                 }
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: max_header_list_size=%u", value);
                 break;
@@ -611,7 +617,7 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
         value = ntohl(*(uint32_t *) buf);
         swoole_http2_frame_trace_log(recv, "window_size_increment=%d", value);
         if (stream_id == 0) {
-            remote_settings.window_size += value;
+            remote_window_size += value;
         } else {
             Stream *stream = get_stream(stream_id);
             if (stream) {
@@ -712,19 +718,19 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
 
             // now we control the connection flow only (not stream)
             // our window size is unlimited, so we don't worry about subtraction overflow
-            local_settings.window_size -= length;
+            local_window_size -= length;
             stream->local_window_size -= length;
-            if (local_settings.window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                if (!send_window_update(0, SW_HTTP2_MAX_WINDOW_SIZE - local_settings.window_size)) {
+            if (local_window_size < (local_settings.init_window_size / 4)) {
+                if (!send_window_update(0, local_settings.init_window_size - local_window_size)) {
                     return SW_ERROR;
                 }
-                local_settings.window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+                local_window_size = local_settings.init_window_size;
             }
-            if (stream->local_window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                if (!send_window_update(stream_id, SW_HTTP2_MAX_WINDOW_SIZE - stream->local_window_size)) {
+            if (stream->local_window_size < (local_settings.init_window_size / 4)) {
+                if (!send_window_update(stream_id, local_settings.init_window_size - stream->local_window_size)) {
                     return SW_ERROR;
                 }
-                stream->local_window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+                stream->local_window_size = local_settings.init_window_size;
             }
         }
     }
@@ -778,10 +784,13 @@ int php_swoole_zlib_decompress(z_stream *stream, String *buffer, char *body, int
     stream->total_in = 0;
     stream->total_out = 0;
 
-#if 0
-    printf(SW_START_LINE"\nstatus=%d\tavail_in=%ld,\tavail_out=%ld,\ttotal_in=%ld,\ttotal_out=%ld\n", status,
-            stream->avail_in, stream->avail_out, stream->total_in, stream->total_out);
-#endif
+    swoole_trace_log(SW_TRACE_ZLIB,
+                     SW_START_LINE "\nstatus=%d\tavail_in=%u,\tavail_out=%u,\ttotal_in=%lu,\ttotal_out=%lu\n",
+                     status,
+                     stream->avail_in,
+                     stream->avail_out,
+                     stream->total_in,
+                     stream->total_out);
 
     buffer->clear();
 
@@ -791,10 +800,15 @@ int php_swoole_zlib_decompress(z_stream *stream, String *buffer, char *body, int
 
         status = inflate(stream, Z_SYNC_FLUSH);
 
-#if 0
-        printf("status=%d\tavail_in=%ld,\tavail_out=%ld,\ttotal_in=%ld,\ttotal_out=%ld,\tlength=%ld\n", status,
-                stream->avail_in, stream->avail_out, stream->total_in, stream->total_out, buffer->length);
-#endif
+        swoole_trace_log(SW_TRACE_ZLIB,
+                         "status=%d\tavail_in=%d,\tavail_out=%d,\ttotal_in=%lu,\ttotal_out=%lu,\tlength=%lu\n",
+                         status,
+                         stream->avail_in,
+                         stream->avail_out,
+                         stream->total_in,
+                         stream->total_out,
+                         buffer->length);
+
         if (status >= 0) {
             buffer->length = stream->total_out;
         }
@@ -887,45 +901,10 @@ bool Client::send_window_update(int stream_id, uint32_t size) {
  * called on connect
  */
 bool Client::send_setting() {
-    Http2::Settings *settings = &local_settings;
-    uint16_t id = 0;
-    uint32_t value = 0;
-
-    char frame[SW_HTTP2_FRAME_HEADER_SIZE + 18];
-    memset(frame, 0, sizeof(frame));
-    Http2::set_frame_header(frame, SW_HTTP2_TYPE_SETTINGS, 18, 0, 0);
-
-    char *p = frame + SW_HTTP2_FRAME_HEADER_SIZE;
-    /**
-     * HEADER_TABLE_SIZE
-     */
-    id = htons(SW_HTTP2_SETTING_HEADER_TABLE_SIZE);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->header_table_size);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-    /**
-     * MAX_CONCURRENT_STREAMS
-     */
-    id = htons(SW_HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->max_concurrent_streams);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-    /**
-     * INIT_WINDOW_SIZE
-     */
-    id = htons(SW_HTTP2_SETTINGS_INIT_WINDOW_SIZE);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->window_size);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-
-    swoole_trace_log(SW_TRACE_HTTP2, "[" SW_ECHO_GREEN "]\t[length=%d]", Http2::get_type(SW_HTTP2_TYPE_SETTINGS), 18);
-    return send(frame, SW_HTTP2_FRAME_HEADER_SIZE + 18);
+    char frame[SW_HTTP2_SETTING_FRAME_SIZE];
+    size_t n = Http2::pack_setting_frame(frame, local_settings, false);
+    swoole_trace_log(SW_TRACE_HTTP2, "[" SW_ECHO_GREEN "]\t[length=%lu]", Http2::get_type(SW_HTTP2_TYPE_SETTINGS), n);
+    return send(frame, n);
 }
 
 void http_parse_set_cookies(const char *at, size_t length, zval *zcookies, zval *zset_cookie_headers);
@@ -1084,7 +1063,7 @@ ssize_t Client::build_header(zval *zobject, zval *zrequest, char *buffer) {
         zend_string *key;
         zval *zvalue;
         char *encoded_value;
-        int encoded_value_len;
+        size_t encoded_value_len;
         String *buffer = sw_tg_buffer();
 
         ZEND_HASH_FOREACH_STR_KEY_VAL(Z_ARRVAL_P(zcookies), key, zvalue) {
@@ -1106,11 +1085,14 @@ ssize_t Client::build_header(zval *zobject, zval *zrequest, char *buffer) {
     }
 
     size_t buflen = nghttp2_hd_deflate_bound(h2c->deflater, headers.get(), headers.len());
-    // if (buflen > h2c->remote_settings.max_header_list_size) {
-    //     php_swoole_error(E_WARNING, "header cannot bigger than remote max_header_list_size %u",
-    //     h2c->remote_settings.max_header_list_size);
-    //     return -1;
-    // }
+#if 0
+    if (buflen > h2c->remote_settings.max_header_list_size) {
+        php_swoole_error(E_WARNING,
+                         "header cannot bigger than remote max_header_list_size %u",
+                         client->remote_settings.max_header_list_size);
+        return -1;
+    }
+#endif
     ssize_t rv = nghttp2_hd_deflate_hd(h2c->deflater, (uchar *) buffer, buflen, headers.get(), headers.len());
     if (rv < 0) {
         h2c->nghttp2_error(rv, "nghttp2_hd_deflate_hd() failed");
@@ -1139,8 +1121,8 @@ Stream *Client::create_stream(uint32_t stream_id, uint8_t flags) {
     // init
     stream->stream_id = stream_id;
     stream->flags = flags;
-    stream->remote_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-    stream->local_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+    stream->remote_window_size = remote_settings.init_window_size;
+    stream->local_window_size = local_settings.init_window_size;
     streams.emplace(stream_id, stream);
     // create response object
     object_init_ex(&stream->zresponse, swoole_http2_response_ce);
@@ -1351,7 +1333,7 @@ bool Client::send_goaway_frame(zend_long error_code, const char *debug_data, siz
         memcpy(frame + SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_GOAWAY_SIZE, debug_data, debug_data_len);
     }
     swoole_trace_log(SW_TRACE_HTTP2,
-                     "[" SW_ECHO_GREEN "] Send: last-sid=%u, error-code=%ld",
+                     "[" SW_ECHO_GREEN "] Send: last-sid=%u, error-code=" ZEND_LONG_FMT,
                      Http2::get_type(SW_HTTP2_TYPE_GOAWAY),
                      last_stream_id,
                      error_code);
@@ -1425,10 +1407,10 @@ static PHP_METHOD(swoole_http2_client_coro, connect) {
     RETURN_BOOL(h2c->connect());
 }
 
-static sw_inline void http2_settings_to_array(Http2::Settings *settings, zval *zarray) {
+static sw_inline void http2_client_settings_to_array(Http2::Settings *settings, zval *zarray) {
     array_init(zarray);
     add_assoc_long_ex(zarray, ZEND_STRL("header_table_size"), settings->header_table_size);
-    add_assoc_long_ex(zarray, ZEND_STRL("window_size"), settings->window_size);
+    add_assoc_long_ex(zarray, ZEND_STRL("init_window_size"), settings->init_window_size);
     add_assoc_long_ex(zarray, ZEND_STRL("max_concurrent_streams"), settings->max_concurrent_streams);
     add_assoc_long_ex(zarray, ZEND_STRL("max_frame_size"), settings->max_frame_size);
     add_assoc_long_ex(zarray, ZEND_STRL("max_header_list_size"), settings->max_header_list_size);
@@ -1438,19 +1420,22 @@ static PHP_METHOD(swoole_http2_client_coro, stats) {
     Client *h2c = php_swoole_get_h2c(ZEND_THIS);
     zval _zarray, *zarray = &_zarray;
     String key = {};
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s", &key.str, &key.length) == FAILURE) {
-        RETURN_FALSE;
-    }
+
+    ZEND_PARSE_PARAMETERS_START(0, 1)
+    Z_PARAM_OPTIONAL
+    Z_PARAM_STRING(key.str, key.length);
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     if (key.length > 0) {
         if (SW_STREQ(key.str, key.length, "current_stream_id")) {
             RETURN_LONG(h2c->stream_id);
         } else if (SW_STREQ(key.str, key.length, "last_stream_id")) {
             RETURN_LONG(h2c->last_stream_id);
         } else if (SW_STREQ(key.str, key.length, "local_settings")) {
-            http2_settings_to_array(&h2c->local_settings, zarray);
+            http2_client_settings_to_array(&h2c->local_settings, zarray);
             RETURN_ZVAL(zarray, 0, 0);
         } else if (SW_STREQ(key.str, key.length, "remote_settings")) {
-            http2_settings_to_array(&h2c->remote_settings, zarray);
+            http2_client_settings_to_array(&h2c->remote_settings, zarray);
             RETURN_ZVAL(zarray, 0, 0);
         } else if (SW_STREQ(key.str, key.length, "active_stream_num")) {
             RETURN_LONG(h2c->streams.size());
@@ -1459,9 +1444,9 @@ static PHP_METHOD(swoole_http2_client_coro, stats) {
         array_init(return_value);
         add_assoc_long_ex(return_value, ZEND_STRL("current_stream_id"), h2c->stream_id);
         add_assoc_long_ex(return_value, ZEND_STRL("last_stream_id"), h2c->last_stream_id);
-        http2_settings_to_array(&h2c->local_settings, zarray);
+        http2_client_settings_to_array(&h2c->local_settings, zarray);
         add_assoc_zval_ex(return_value, ZEND_STRL("local_settings"), zarray);
-        http2_settings_to_array(&h2c->remote_settings, zarray);
+        http2_client_settings_to_array(&h2c->remote_settings, zarray);
         add_assoc_zval_ex(return_value, ZEND_STRL("remote_settings"), zarray);
         add_assoc_long_ex(return_value, ZEND_STRL("active_stream_num"), h2c->streams.size());
     }
@@ -1496,9 +1481,14 @@ static PHP_METHOD(swoole_http2_client_coro, write) {
     zend_long stream_id;
     zval *data;
     zend_bool end = 0;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "lz|b", &stream_id, &data, &end) == FAILURE) {
-        RETURN_FALSE;
-    }
+
+    ZEND_PARSE_PARAMETERS_START(2, 3)
+    Z_PARAM_LONG(stream_id);
+    Z_PARAM_ZVAL(data);
+    Z_PARAM_OPTIONAL
+    Z_PARAM_BOOL(end);
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     RETURN_BOOL(h2c->write_data(stream_id, data, end));
 }
 
@@ -1527,7 +1517,7 @@ static PHP_METHOD(swoole_http2_client_coro, ping) {
  */
 static PHP_METHOD(swoole_http2_client_coro, goaway) {
     Client *h2c = php_swoole_get_h2c(ZEND_THIS);
-    zend_long error_code = SW_HTTP2_ERROR_NO_ERROR;
+    zend_long h2_error_code = SW_HTTP2_ERROR_NO_ERROR;
     char *debug_data = nullptr;
     size_t debug_data_len = 0;
 
@@ -1535,11 +1525,13 @@ static PHP_METHOD(swoole_http2_client_coro, goaway) {
         RETURN_FALSE;
     }
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "|ls", &error_code, &debug_data, &debug_data_len) == FAILURE) {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START(0, 2)
+    Z_PARAM_OPTIONAL
+    Z_PARAM_LONG(h2_error_code);
+    Z_PARAM_STRING(debug_data, debug_data_len);
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    RETURN_BOOL(h2c->send_goaway_frame(error_code, debug_data, debug_data_len));
+    RETURN_BOOL(h2c->send_goaway_frame(h2_error_code, debug_data, debug_data_len));
 }
 
 #endif
