@@ -18,6 +18,8 @@
 
 #ifdef SW_USE_HTTP2
 
+#include <sstream>
+
 #include "swoole_static_handler.h"
 
 #include "main/php_variables.h"
@@ -25,6 +27,7 @@
 using namespace swoole;
 using std::string;
 using swoole::coroutine::System;
+using swoole::http2::get_default_setting;
 using swoole::http_server::StaticHandler;
 
 namespace Http2 = swoole::http2;
@@ -37,6 +40,7 @@ static std::unordered_map<SessionId, Http2Session *> http2_sessions;
 extern String *swoole_http_buffer;
 
 static bool http2_server_respond(HttpContext *ctx, const String *body);
+static bool http2_server_send_range_file(HttpContext *ctx, swoole::http_server::StaticHandler *handler);
 
 Http2Stream::Stream(Http2Session *client, uint32_t _id) {
     ctx = swoole_http_context_new(client->fd);
@@ -45,8 +49,8 @@ Http2Stream::Stream(Http2Session *client, uint32_t _id) {
     ctx->stream = this;
     ctx->keepalive = true;
     id = _id;
-    local_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-    remote_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+    local_window_size = client->local_settings.init_window_size;
+    remote_window_size = client->remote_settings.init_window_size;
 }
 
 Http2Stream::~Stream() {
@@ -69,6 +73,8 @@ Http2Session::Session(SessionId _fd) {
     Http2::init_settings(&local_settings);
     // [init]: we must set default value, peer is not always send all the settings
     Http2::init_settings(&remote_settings);
+    local_window_size = local_settings.init_window_size;
+    remote_window_size = remote_settings.init_window_size;
     last_stream_id = 0;
     shutting_down = false;
     is_coro = false;
@@ -134,13 +140,14 @@ static ssize_t http2_server_build_trailer(HttpContext *ctx, uchar *buffer) {
         }
 
         buflen = nghttp2_hd_deflate_bound(deflater, trailer.get(), trailer.len());
-        /*
-        if (buflen > SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE)
-        {
-            php_swoole_error(E_WARNING, "header cannot bigger than remote max_header_list_size %u",
-        SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE); return -1;
+#if 0
+        if (buflen > SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE) {
+            php_swoole_error(E_WARNING,
+                             "header cannot bigger than remote max_header_list_size %u",
+                             SW_HTTP2_DEFAULT_MAX_HEADER_LIST_SIZE);
+            return -1;
         }
-        */
+#endif
         rv = nghttp2_hd_deflate_hd(deflater, (uchar *) buffer, buflen, trailer.get(), trailer.len());
         if (rv < 0) {
             swoole_warning("nghttp2_hd_deflate_hd() failed with error: %s", nghttp2_strerror((int) rv));
@@ -189,8 +196,8 @@ static bool http2_server_is_static_file(Server *serv, HttpContext *ctx) {
         auto date_str = handler.get_date();
         auto date_str_last_modified = handler.get_date_last_modified();
 
-        zval *zheader = ctx->request.zserver;
-        ctx->set_header(ZEND_STRL("Last-Modified"), date_str.c_str(), date_str.length(), 0);
+        zval *zheader = ctx->request.zheader;
+        ctx->set_header(ZEND_STRL("Last-Modified"), date_str_last_modified.c_str(), date_str_last_modified.length(), 0);
 
         zval *zdate_if_modified_since = zend_hash_str_find(Z_ARR_P(zheader), ZEND_STRL("if-modified-since"));
         if (zdate_if_modified_since) {
@@ -201,14 +208,36 @@ static bool http2_server_is_static_file(Server *serv, HttpContext *ctx) {
             }
         }
 
-        zend::String _filename(handler.get_filename_std_string());
-        zval zfilename;
-        ZVAL_STR(&zfilename, _filename.get());
-        zval retval; /* do not care the retval (the connection will be closed if failed) */
-        sw_zend_call_method_with_1_params(
-            ctx->response.zobject, swoole_http_response_ce, nullptr, "sendfile", &retval, &zfilename);
+        zval *zrange = zend_hash_str_find(Z_ARR_P(zheader), ZEND_STRL("range"));
+        zval *zif_range = zend_hash_str_find(Z_ARR_P(zheader), ZEND_STRL("if-range"));
+        handler.parse_range(zrange ? Z_STRVAL_P(zrange) : nullptr, zif_range ? Z_STRVAL_P(zif_range) : nullptr);
+        ctx->response.status = handler.status_code;
+        auto tasks = handler.get_tasks();
+        if (1 == tasks.size()) {
+            if (0 == tasks[0].offset && tasks[0].length == handler.get_filesize()) {
+                ctx->set_header(ZEND_STRL("Accept-Ranges"), SW_STRL("bytes"), 0);
+            } else {
+                std::stringstream content_range;
+                content_range << "bytes";
+                if (tasks[0].length != handler.get_filesize()) {
+                    content_range << " " << tasks[0].offset << "-" << (tasks[0].length + tasks[0].offset - 1) << "/"
+                                  << handler.get_filesize();
+                }
+                auto content_range_str = content_range.str();
+                ctx->set_header(ZEND_STRL("Content-Range"), content_range_str.c_str(), content_range_str.length(), 0);
+            }
+        }
 
-        return true;
+        // request_method
+        zval *zrequest_method = zend_hash_str_find(Z_ARR_P(zserver), ZEND_STRL("request_method"));
+        if (zrequest_method && Z_TYPE_P(zrequest_method) == IS_STRING &&
+            SW_STRCASEEQ(Z_STRVAL_P(zrequest_method), Z_STRLEN_P(zrequest_method), "HEAD")) {
+            String empty_body;
+            http2_server_respond(ctx, &empty_body);
+            return true;
+        } else {
+            return http2_server_send_range_file(ctx, &handler);
+        }
     }
 
     return false;
@@ -218,17 +247,23 @@ static void http2_server_onRequest(Http2Session *client, Http2Stream *stream) {
     HttpContext *ctx = stream->ctx;
     zval *zserver = ctx->request.zserver;
     Server *serv = (Server *) ctx->private_data;
+    zval args[2];
+    zend_fcall_info_cache *fci_cache = nullptr;
+    Connection *serv_sock = nullptr;
+    int server_fd = 0;
 
     Connection *conn = serv->get_connection_by_session_id(ctx->fd);
-    int server_fd = conn->server_fd;
-    Connection *serv_sock = serv->get_connection(server_fd);
+    if (!conn) {
+        goto _destroy;
+    }
+
+    server_fd = conn->server_fd;
+    serv_sock = serv->get_connection(server_fd);
 
     ctx->request.version = SW_HTTP_VERSION_2;
 
     if (serv->enable_static_handler && http2_server_is_static_file(serv, ctx)) {
-        zval_ptr_dtor(ctx->request.zobject);
-        zval_ptr_dtor(ctx->response.zobject);
-        return;
+        goto _destroy;
     }
 
     add_assoc_long(zserver, "request_time", time(nullptr));
@@ -241,15 +276,17 @@ static void http2_server_onRequest(Http2Session *client, Http2Stream *stream) {
     add_assoc_long(zserver, "master_time", conn->last_recv_time);
     add_assoc_string(zserver, "server_protocol", (char *) "HTTP/2");
 
-    zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onRequest);
-    zval args[2] = {*ctx->request.zobject, *ctx->response.zobject};
+    fci_cache = php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onRequest);
+    args[0] = *ctx->request.zobject;
+    args[1] = *ctx->response.zobject;
     if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, nullptr, serv->is_enable_coroutine()))) {
         stream->reset(SW_HTTP2_ERROR_INTERNAL_ERROR);
         php_swoole_error(E_WARNING, "%s->onRequest[v2] handler error", ZSTR_VAL(swoole_http_server_ce->name));
     }
 
-    zval_ptr_dtor(&args[0]);
-    zval_ptr_dtor(&args[1]);
+_destroy:
+    zval_ptr_dtor(ctx->request.zobject);
+    zval_ptr_dtor(ctx->response.zobject);
 }
 
 static void http2_server_set_date_header(Http2::HeaderSet *headers) {
@@ -491,7 +528,7 @@ bool Http2Stream::send_body(const String *body, bool end_stream, size_t max_fram
         http2::set_frame_header(frame_header, SW_HTTP2_TYPE_DATA, send_n, _send_flags, id);
 
         // send twice to reduce memory copy
-        if (send_n < SwooleG.pagesize) {
+        if (send_n < swoole_pagesize()) {
             swoole_http_buffer->clear();
             swoole_http_buffer->append(frame_header, SW_HTTP2_FRAME_HEADER_SIZE);
             swoole_http_buffer->append(p, send_n);
@@ -570,7 +607,7 @@ static bool http2_server_respond(HttpContext *ctx, const String *body) {
 
     // If send_yield is not supported, ignore flow control
     if (ctx->co_socket || !((Server *) ctx->private_data)->send_yield || !swoole_coroutine_is_in()) {
-        if (body->length > client->remote_settings.window_size) {
+        if (body->length > client->remote_window_size) {
             swoole_warning("The data sent exceeded remote_window_size");
         }
         if (!stream->send_body(body, end_stream, client->local_settings.max_frame_size)) {
@@ -635,6 +672,126 @@ static bool http2_server_respond(HttpContext *ctx, const String *body) {
     return !error;
 }
 
+static bool http2_server_send_range_file(HttpContext *ctx, swoole::http_server::StaticHandler *handler) {
+    Http2Session *client = http2_sessions[ctx->fd];
+    std::shared_ptr<String> body;
+
+#ifdef SW_HAVE_COMPRESSION
+    ctx->accept_compression = 0;
+#endif
+    bool error = false;
+    zval *ztrailer =
+        sw_zend_read_property_ex(swoole_http_response_ce, ctx->response.zobject, SW_ZSTR_KNOWN(SW_ZEND_STR_TRAILER), 0);
+    if (php_swoole_array_length_safe(ztrailer) == 0) {
+        ztrailer = nullptr;
+    }
+    zval *zheader =
+        sw_zend_read_and_convert_property_array(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("header"), 0);
+    if (!zend_hash_str_exists(Z_ARRVAL_P(zheader), ZEND_STRL("content-type"))) {
+        ctx->set_header(ZEND_STRL("content-type"), handler->get_content_type(), strlen(handler->get_content_type()), 0);
+    }
+
+    bool end_stream = (ztrailer == nullptr);
+    if (!ctx->stream->send_header(handler->get_content_length(), end_stream)) {
+        return false;
+    }
+
+    /* headers has already been sent, retries are no longer allowed (even if send body failed) */
+    ctx->end_ = 1;
+
+    auto tasks = handler->get_tasks();
+    if (!tasks.empty()) {
+        File fp(handler->get_filename(), O_RDONLY);
+        if (!fp.ready()) {
+            return false;
+        }
+
+        char *buf;
+        if (tasks.size() > 1) {
+            for (auto i = tasks.begin(); i != tasks.end(); i++) {
+                body.reset(new String(i->part_header, strlen(i->part_header)));
+                if (!ctx->stream->send_body(
+                        body.get(), false, client->local_settings.max_frame_size, 0, body->length)) {
+                    error = true;
+                    break;
+                } else {
+                    client->remote_window_size -= body->length;  // TODO: flow control?
+                }
+
+                fp.set_offest(i->offset);
+                buf = (char *) emalloc(i->length);
+                auto n_reads = fp.read(buf, i->length);
+                if (n_reads < 0) {
+                    efree(buf);
+                    return false;
+                }
+                body.reset(new String(buf, i->length));
+                efree(buf);
+                if (!ctx->stream->send_body(
+                        body.get(), false, client->local_settings.max_frame_size, 0, body->length)) {
+                    error = true;
+                    break;
+                } else {
+                    client->remote_window_size -= body->length;  // TODO: flow control?
+                }
+            }
+
+            if (!error) {
+                body.reset(new String(handler->get_end_part(), strlen(handler->get_end_part())));
+                if (!ctx->stream->send_body(
+                        body.get(), end_stream, client->local_settings.max_frame_size, 0, body->length)) {
+                    error = true;
+                } else {
+                    client->remote_window_size -= body->length;  // TODO: flow control?
+                }
+            }
+        } else if (tasks[0].length > 0) {
+            auto callback = [&]() -> bool {
+                fp.set_offest(tasks[0].offset);
+                buf = (char *) emalloc(tasks[0].length);
+                auto n_reads = fp.read(buf, tasks[0].length);
+                if (n_reads < 0) {
+                    efree(buf);
+                    return false;
+                }
+                body.reset(new String(buf, n_reads));
+                efree(buf);
+                return true;
+            };
+            if (swoole_coroutine_is_in()) {
+                if (!swoole::coroutine::async(callback)) {
+                    return false;
+                }
+            } else {
+                if (!callback()) {
+                    return false;
+                }
+            }
+            if (!ctx->stream->send_body(
+                    body.get(), end_stream, client->local_settings.max_frame_size, 0, body->length)) {
+                error = true;
+            } else {
+                client->remote_window_size -= body->length;  // TODO: flow control?
+            }
+        }
+    }
+
+    if (!error && ztrailer) {
+        if (!ctx->stream->send_trailer()) {
+            error = true;
+        }
+    }
+
+    if (error) {
+        ctx->close(ctx);
+    } else {
+        client->streams.erase(ctx->stream->id);
+        delete ctx->stream;
+    }
+
+    return true;
+}
+
 bool HttpContext::http2_send_file(const char *file, uint32_t l_file, off_t offset, size_t length) {
     Http2Session *client = http2_sessions[fd];
     std::shared_ptr<String> body;
@@ -657,9 +814,6 @@ bool HttpContext::http2_send_file(const char *file, uint32_t l_file, off_t offse
             return false;
         }
         body = fp.read_content();
-        if (body->empty()) {
-            return false;
-        }
     }
     body->length = SW_MIN(length, body->length);
 
@@ -690,7 +844,7 @@ bool HttpContext::http2_send_file(const char *file, uint32_t l_file, off_t offse
         if (!stream->send_body(body.get(), end_stream, client->local_settings.max_frame_size, offset, length)) {
             error = true;
         } else {
-            client->remote_settings.window_size -= length;  // TODO: flow control?
+            client->remote_window_size -= length;  // TODO: flow control?
         }
     }
 
@@ -891,7 +1045,7 @@ int swoole_http2_server_parse(Http2Session *client, const char *buf) {
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: max_concurrent_streams=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_INIT_WINDOW_SIZE:
-                client->remote_settings.window_size = value;
+                client->remote_window_size = client->remote_settings.init_window_size = value;
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: init_window_size=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_MAX_FRAME_SIZE:
@@ -962,17 +1116,19 @@ int swoole_http2_server_parse(Http2Session *client, const char *buf) {
         buffer->append(buf, length);
 
         // flow control
-        client->local_settings.window_size -= length;
+        client->local_window_size -= length;
         stream->local_window_size -= length;
 
         if (length > 0) {
-            if (client->local_settings.window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                http2_server_send_window_update(ctx, 0, SW_HTTP2_MAX_WINDOW_SIZE - client->local_settings.window_size);
-                client->local_settings.window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+            if (client->local_window_size < (client->local_settings.init_window_size / 4)) {
+                http2_server_send_window_update(
+                    ctx, 0, client->local_settings.init_window_size - client->local_window_size);
+                client->local_window_size = client->local_settings.init_window_size;
             }
-            if (stream->local_window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                http2_server_send_window_update(ctx, stream_id, SW_HTTP2_MAX_WINDOW_SIZE - stream->local_window_size);
-                stream->local_window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+            if (stream->local_window_size < (client->local_settings.init_window_size / 4)) {
+                http2_server_send_window_update(
+                    ctx, stream_id, client->local_settings.init_window_size - stream->local_window_size);
+                stream->local_window_size = client->local_settings.init_window_size;
             }
         }
 
@@ -1010,7 +1166,7 @@ int swoole_http2_server_parse(Http2Session *client, const char *buf) {
     case SW_HTTP2_TYPE_WINDOW_UPDATE: {
         value = ntohl(*(uint32_t *) buf);
         if (stream_id == 0) {
-            client->remote_settings.window_size += value;
+            client->remote_window_size += value;
         } else {
             if (client->streams.find(stream_id) != client->streams.end()) {
                 stream = client->streams[stream_id];
