@@ -346,6 +346,17 @@ static bool parse_header_flags(HttpContext *ctx, const char *key, size_t keylen,
     } else if (SW_STRCASEEQ(key, keylen, "Date")) {
         header_flags |= HTTP_HEADER_DATE;
     } else if (SW_STRCASEEQ(key, keylen, "Content-Length")) {
+        // https://github.com/swoole/swoole-src/issues/4857
+#ifdef SW_HAVE_COMPRESSION
+        if (ctx->accept_compression) {
+            php_swoole_error(E_WARNING, "The client has set 'Accept-Encoding', 'Content-Length' is ignored");
+            return false;
+        }
+#endif
+        if (ctx->send_chunked) {
+            php_swoole_error(E_WARNING, "You have set 'Transfer-Encoding', 'Content-Length' is ignored");
+            return false;
+        }
         header_flags |= HTTP_HEADER_CONTENT_LENGTH;
     } else if (SW_STRCASEEQ(key, keylen, "Content-Type")) {
         header_flags |= HTTP_HEADER_CONTENT_TYPE;
@@ -634,7 +645,7 @@ int swoole_http_response_compress(const char *data, size_t length, int method, i
         level = Z_BEST_COMPRESSION;
     }
 
-    size_t memory_size = ((size_t)((double) length * (double) 1.015)) + 10 + 8 + 4 + 1;
+    size_t memory_size = ((size_t) ((double) length * (double) 1.015)) + 10 + 8 + 4 + 1;
     if (memory_size > swoole_zlib_buffer->size) {
         if (!swoole_zlib_buffer->extend(memory_size)) {
             return SW_ERR;
@@ -1010,7 +1021,7 @@ static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const
     } else {
         if (url_encode) {
             char *encoded_value;
-            int encoded_value_len;
+            size_t encoded_value_len;
             encoded_value = php_swoole_url_encode(value, value_len, &encoded_value_len);
             cookie_size += encoded_value_len;
             cookie = (char *) emalloc(cookie_size);
