@@ -112,7 +112,7 @@ int php_swoole_http_server_onReceive(Server *serv, RecvData *req) {
         zend_fcall_info_cache *fci_cache = nullptr;
 
         if (conn->websocket_status == WebSocket::STATUS_CONNECTION) {
-            fci_cache = php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onHandShake);
+            fci_cache = php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onHandshake);
             if (fci_cache == nullptr) {
                 swoole_websocket_onHandshake(serv, port, ctx);
                 goto _dtor_and_return;
@@ -311,6 +311,7 @@ bool http_context_send_data(HttpContext *ctx, const char *data, size_t length) {
         zval yield_data, return_value;
         ZVAL_STRINGL(&yield_data, data, length);
         php_swoole_server_send_yield(serv, ctx->fd, &yield_data, &return_value);
+        zval_ptr_dtor(&yield_data);
         return Z_BVAL_P(&return_value);
     }
     return retval;
