@@ -953,6 +953,7 @@ void mysql_client::handle_strict_type(zval *ztext, mysql::field_packet *field) {
         case SW_MYSQL_TYPE_VAR_STRING:
         case SW_MYSQL_TYPE_VARCHAR:
         case SW_MYSQL_TYPE_NEWDATE:
+        case SW_MYSQL_TYPE_GEOMETRY:
         /* Date Time */
         case SW_MYSQL_TYPE_TIME:
         case SW_MYSQL_TYPE_YEAR:
@@ -1398,7 +1399,8 @@ void mysql_statement::fetch(zval *return_value) {
             case SW_MYSQL_TYPE_STRING:
             case SW_MYSQL_TYPE_VAR_STRING:
             case SW_MYSQL_TYPE_VARCHAR:
-            case SW_MYSQL_TYPE_NEWDATE: {
+            case SW_MYSQL_TYPE_NEWDATE:
+            case SW_MYSQL_TYPE_GEOMETRY: {
             _add_string:
                 zval ztext;
                 client->handle_row_data_text(&ztext, &row_data, field);
@@ -1490,7 +1492,7 @@ void mysql_statement::fetch(zval *return_value) {
                     }
                     break;
                 case SW_MYSQL_TYPE_FLOAT: {
-                    double dv = sw_php_math_round(*(float *) p, 5, PHP_ROUND_HALF_DOWN);
+                    double dv = sw_php_math_round(*(float *) p, 7, PHP_ROUND_HALF_DOWN);
                     add_assoc_double_ex(return_value, field->name, field->name_length, dv);
                     swoole_trace_log(SW_TRACE_MYSQL_CLIENT, "%.*s=%.7f", field->name_length, field->name, dv);
                 } break;
@@ -1695,6 +1697,9 @@ void php_swoole_mysql_coro_minit(int module_number) {
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_mysql_coro, sw_zend_class_unset_property_deny);
     SW_SET_CLASS_CUSTOM_OBJECT(
         swoole_mysql_coro, php_swoole_mysql_coro_create_object, php_swoole_mysql_coro_free_object, mysql_coro_t, std);
+#if PHP_VERSION_ID >= 80200
+	zend_add_parameter_attribute((zend_function *) zend_hash_str_find_ptr(&swoole_mysql_coro_ce->function_table, SW_STRL("connect")), 0, ZSTR_KNOWN(ZEND_STR_SENSITIVEPARAMETER), 0);
+#endif
 
     SW_INIT_CLASS_ENTRY(swoole_mysql_coro_statement,
                         "Swoole\\Coroutine\\MySQL\\Statement",
