@@ -10,7 +10,7 @@
  | to obtain it through the world-wide-web, please send a note to       |
  | license@swoole.com so we can mail you a copy immediately.            |
  +----------------------------------------------------------------------+
- | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+ | Author: Tianfeng Han  <rango@swoole.com>                             |
  +----------------------------------------------------------------------+
  */
 
@@ -41,6 +41,7 @@
 #include "swoole_async.h"
 #include "swoole_c_api.h"
 #include "swoole_coroutine_c_api.h"
+#include "swoole_coroutine_system.h"
 #include "swoole_ssl.h"
 
 #if defined(__APPLE__) && defined(HAVE_CCRANDOMGENERATEBYTES)
@@ -54,6 +55,7 @@
 #endif
 
 using swoole::String;
+using swoole::coroutine::System;
 
 #ifdef HAVE_GETRANDOM
 #include <sys/random.h>
@@ -291,6 +293,20 @@ SW_API void swoole_set_log_level(int level) {
     }
 }
 
+SW_API int swoole_get_log_level() {
+    if (sw_logger()) {
+        return sw_logger()->get_level();
+    } else {
+        return SW_LOG_NONE;
+    }
+}
+
+SW_API void swoole_set_log_file(const char *file) {
+    if (sw_logger()) {
+        sw_logger()->open(file);
+    }
+}
+
 SW_API void swoole_set_trace_flags(int flags) {
     SwooleG.trace_flags = flags;
 }
@@ -539,7 +555,7 @@ ulong_t swoole_hex2dec(const char *hex, size_t *parsed_bytes) {
 #endif
 
 int swoole_rand(int min, int max) {
-    static int _seed = 0;
+    static time_t _seed = 0;
     assert(max > min);
 
     if (_seed == 0) {
@@ -773,12 +789,13 @@ char *swoole_string_format(size_t n, const char *format, ...) {
     return nullptr;
 }
 
+static const char characters[] = {
+    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U',
+    'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
+    'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
+};
+
 void swoole_random_string(char *buf, size_t size) {
-    static char characters[] = {
-        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U',
-        'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
-        'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-    };
     size_t i = 0;
     for (; i < size; i++) {
         buf[i] = characters[swoole_rand(0, sizeof(characters) - 1)];
@@ -786,6 +803,13 @@ void swoole_random_string(char *buf, size_t size) {
     buf[i] = '\0';
 }
 
+void swoole_random_string(std::string &str, size_t size) {
+    size_t i = 0;
+    for (; i < size; i++) {
+        str.append(1, characters[swoole_rand(0, sizeof(characters) - 1)]);
+    }
+}
+
 size_t swoole_random_bytes(char *buf, size_t size) {
     size_t read_bytes = 0;
     ssize_t n;
@@ -871,7 +895,7 @@ size_t DataHead::dump(char *_buf, size_t _len) {
                        "DataHead[%p]\n"
                        "{\n"
                        "    long fd = %ld;\n"
-                       "    uint64_t msg_id = %lu;\n"
+                       "    uint64_t msg_id = %" PRIu64 ";\n"
                        "    uint32_t len = %d;\n"
                        "    int16_t reactor_id = %d;\n"
                        "    uint8_t type = %d;\n"
