@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 
@@ -31,6 +31,9 @@
 #include <sys/event.h>
 #endif
 #endif
+#ifdef __NetBSD__
+#include <sys/param.h>
+#endif
 
 using swoole::Event;
 using swoole::Reactor;
@@ -142,7 +145,7 @@ static void swoole_signal_async_handler(int signo) {
         sw_reactor()->singal_no = signo;
     } else {
         // discard signal
-        if (_lock) {
+        if (_lock || !SwooleG.init) {
             return;
         }
         _lock = 1;
@@ -181,8 +184,7 @@ void swoole_signal_clear(void) {
     } else
 #endif
     {
-        int i;
-        for (i = 0; i < SW_SIGNO_MAX; i++) {
+        SW_LOOP_N(SW_SIGNO_MAX) {
             if (signals[i].activated) {
 #ifdef HAVE_KQUEUE
                 if (signals[i].signo != SIGCHLD && sw_reactor()) {
@@ -349,7 +351,7 @@ static SignalHandler swoole_signal_kqueue_set(int signo, SignalHandler handler)
         signals[signo].handler = handler;
         signals[signo].signo = signo;
         signals[signo].activated = true;
-#ifndef __NetBSD__
+#if !defined(__NetBSD__) || (defined(__NetBSD__) && __NetBSD_Version__ >= 1000000000)
         auto sigptr = &signals[signo];
 #else
         auto sigptr = reinterpret_cast<intptr_t>(&signals[signo]);
