@@ -142,7 +142,7 @@ pid_t System::waitpid(pid_t __pid, int *__stat_loc, int __options, double timeou
     TimerNode *timer = nullptr;
     if (timeout > 0) {
         timer = swoole_timer_add(
-            timeout * 1000,
+            timeout,
             false,
             [](Timer *timer, TimerNode *tnode) {
                 Coroutine *co = (Coroutine *) tnode->data;
