@@ -42,7 +42,7 @@ long BIO_ctrl(BIO *b, int cmd, long lval, void *ptrval) {
     Session *session = (Session *) BIO_get_data(b);
 
     swoole_trace_log(
-        SW_TRACE_SSL, "BIO_ctrl(BIO[0x%016lX], cmd[%d], lval[%ld], ptrval[0x%016lX])", b, cmd, lval, ptrval);
+        SW_TRACE_SSL, "BIO_ctrl(BIO[0x%016lX], cmd[%d], lval[%ld], ptrval[0x%016lX])", *(long *) b, cmd, lval, *(long *) ptrval);
 
     switch (cmd) {
     case BIO_CTRL_EOF:
@@ -102,7 +102,7 @@ int BIO_create(BIO *b) {
 }
 
 int BIO_destroy(BIO *b) {
-    swoole_trace_log(SW_TRACE_SSL, "BIO_destroy(BIO[0x%016lX])\n", b);
+    swoole_trace_log(SW_TRACE_SSL, "BIO_destroy(BIO[0x%016lX])\n", *(long *) b);
     return 1;
 }
 
