@@ -18,6 +18,7 @@
 #include "swoole_server.h"
 
 #include <string>
+#include <sstream>
 
 #include "swoole_util.h"
 #include "swoole_http2.h"
@@ -73,7 +74,7 @@ bool Server::select_static_handler(http_server::Request *request, Connection *co
     auto date_str = handler.get_date();
     auto date_str_last_modified = handler.get_date_last_modified();
 
-    string date_if_modified_since = request->get_date_if_modified_since();
+    string date_if_modified_since = request->get_header("If-Modified-Since");
     if (!date_if_modified_since.empty() && handler.is_modified(date_if_modified_since)) {
         response.info.len = sw_snprintf(header_buffer,
                                         sizeof(header_buffer),
@@ -131,22 +132,42 @@ bool Server::select_static_handler(http_server::Request *request, Connection *co
         return true;
     }
 
-    auto task = handler.get_task();
-    response.info.len = sw_snprintf(header_buffer,
-                                    sizeof(header_buffer),
-                                    "HTTP/1.1 200 OK\r\n"
-                                    "Connection: %s\r\n"
-                                    "Content-Length: %ld\r\n"
-                                    "Content-Type: %s\r\n"
-                                    "Date: %s\r\n"
-                                    "Last-Modified: %s\r\n"
-                                    "Server: %s\r\n\r\n",
-                                    request->keep_alive ? "keep-alive" : "close",
-                                    (long) task->length,
-                                    handler.get_mimetype(),
-                                    date_str.c_str(),
-                                    date_str_last_modified.c_str(),
-                                    SW_HTTP_SERVER_SOFTWARE);
+    handler.parse_range(request->get_header("Range").c_str(), request->get_header("If-Range").c_str());
+    auto tasks = handler.get_tasks();
+
+    std::stringstream header_stream;
+    if (1 == tasks.size()) {
+        if (0 == tasks[0].offset && tasks[0].length == handler.get_filesize()) {
+            header_stream << "Accept-Ranges: bytes\r\n";
+        } else {
+            header_stream << "Content-Range: bytes";
+            if (tasks[0].length != handler.get_filesize()) {
+                header_stream << " " << tasks[0].offset << "-" << (tasks[0].length + tasks[0].offset - 1) << "/"
+                            << handler.get_filesize();
+            }
+            header_stream << "\r\n";
+        }
+    }
+
+    response.info.len =
+        sw_snprintf(header_buffer,
+                    sizeof(header_buffer),
+                    "HTTP/1.1 %s\r\n"
+                    "Connection: %s\r\n"
+                    "Content-Length: %ld\r\n"
+                    "Content-Type: %s\r\n"
+                    "%s"
+                    "Date: %s\r\n"
+                    "Last-Modified: %s\r\n"
+                    "Server: %s\r\n\r\n",
+                    http_server::get_status_message(handler.status_code),
+                    request->keep_alive ? "keep-alive" : "close",
+                    SW_HTTP_HEAD == request->method ? 0 : handler.get_content_length(),
+                    SW_HTTP_HEAD == request->method ? handler.get_mimetype() : handler.get_content_type(),
+                    header_stream.str().c_str(),
+                    date_str.c_str(),
+                    date_str_last_modified.c_str(),
+                    SW_HTTP_SERVER_SOFTWARE);
 
     response.data = header_buffer;
 
@@ -157,11 +178,40 @@ bool Server::select_static_handler(http_server::Request *request, Connection *co
     send_to_connection(&response);
 
     // Send HTTP body
-    if (task->length != 0) {
-        response.info.type = SW_SERVER_EVENT_SEND_FILE;
-        response.info.len = sizeof(*task) + task->length + 1;
-        response.data = (char *) task;
-        send_to_connection(&response);
+    if (SW_HTTP_HEAD != request->method) {
+        if (!tasks.empty()) {
+            size_t task_size = sizeof(network::SendfileTask) + strlen(handler.get_filename()) + 1;
+            network::SendfileTask *task = (network::SendfileTask *) sw_malloc(task_size);
+            strcpy(task->filename, handler.get_filename());
+            if (tasks.size() > 1) {
+                for (auto i = tasks.begin(); i != tasks.end(); i++) {
+                    response.info.type = SW_SERVER_EVENT_SEND_DATA;
+                    response.info.len = strlen(i->part_header);
+                    response.data = i->part_header;
+                    send_to_connection(&response);
+
+                    task->offset = i->offset;
+                    task->length = i->length;
+                    response.info.type = SW_SERVER_EVENT_SEND_FILE;
+                    response.info.len = task_size;
+                    response.data = (char *) task;
+                    send_to_connection(&response);
+                }
+
+                response.info.type = SW_SERVER_EVENT_SEND_DATA;
+                response.info.len = strlen(handler.get_end_part());
+                response.data = handler.get_end_part();
+                send_to_connection(&response);
+            } else if (tasks[0].length > 0) {
+                task->offset = tasks[0].offset;
+                task->length = tasks[0].length;
+                response.info.type = SW_SERVER_EVENT_SEND_FILE;
+                response.info.len = task_size;
+                response.data = (char *) task;
+                send_to_connection(&response);
+            }
+            sw_free(task);
+        }
     }
 
     // Close the connection if keepalive is not used
@@ -514,15 +564,13 @@ void Request::parse_header_info() {
     for (; p < pe; p++) {
         if (*(p - 1) == '\n' && *(p - 2) == '\r') {
             if (SW_STRCASECT(p, pe - p, "Content-Length:")) {
-                unsigned long long content_length;
                 // strlen("Content-Length:")
                 p += (sizeof("Content-Length:") - 1);
                 // skip spaces
                 while (*p == ' ') {
                     p++;
                 }
-                content_length = strtoull(p, nullptr, 10);
-                content_length_ = SW_MIN(content_length, UINT32_MAX);
+                content_length_ = strtoull(p, nullptr, 10);
                 known_length = 1;
             } else if (SW_STRCASECT(p, pe - p, "Connection:")) {
                 // strlen("Connection:")
@@ -559,13 +607,13 @@ bool Request::has_expect_header() {
     // char *buf = buffer->str + buffer->offset;
     char *buf = buffer_->str;
     // int len = buffer->length - buffer->offset;
-    int len = buffer_->length;
+    size_t len = buffer_->length;
 
     char *pe = buf + len;
     char *p;
 
     for (p = buf; p < pe; p++) {
-        if (*p == '\r' && pe - p > sizeof("\r\nExpect")) {
+        if (*p == '\r' && (size_t)(pe - p) > sizeof("\r\nExpect")) {
             p += 2;
             if (SW_STRCASECT(p, pe - p, "Expect: ")) {
                 p += sizeof("Expect: ") - 1;
@@ -633,34 +681,50 @@ int Request::get_chunked_body_length() {
     return SW_OK;
 }
 
-string Request::get_date_if_modified_since() {
+std::string Request::get_header(const char *name) {
+    size_t name_len = strlen(name);
     char *p = buffer_->str + url_offset_ + url_length_ + 10;
     char *pe = buffer_->str + header_length_;
 
-    string result;
-
-    char *date_if_modified_since = nullptr;
-    size_t length_if_modified_since = 0;
+    char *buffer = nullptr;
+    char *colon = nullptr;
 
     int state = 0;
+    int i = 0;
+
+    bool is_error_header_name = false;
+
     for (; p < pe; p++) {
         switch (state) {
         case 0:
-            if (SW_STRCASECT(p, pe - p, "If-Modified-Since")) {
-                p += sizeof("If-Modified-Since");
+            if (SW_STRCASECT(p, pe - p, "\r\n")) {
+                i = 0;
+                is_error_header_name = false;
+                break;
+            }
+
+            if (!is_error_header_name && swoole_strcasect(p, pe - p, name, name_len)) {
+                colon = p + name_len;
+                if (colon[0] != ':' || i > 1) {
+                    is_error_header_name = true;
+                    break;
+                }
+
+                p += name_len;
                 state = 1;
             }
+
+            i++;
             break;
         case 1:
             if (!isspace(*p)) {
-                date_if_modified_since = p;
+                buffer = p;
                 state = 2;
             }
             break;
         case 2:
             if (SW_STRCASECT(p, pe - p, "\r\n")) {
-                length_if_modified_since = p - date_if_modified_since;
-                return string(date_if_modified_since, length_if_modified_since);
+                return string(buffer, p - buffer);
             }
             break;
         default:
@@ -668,7 +732,7 @@ string Request::get_date_if_modified_since() {
         }
     }
 
-    return string("");
+    return string();
 }
 
 int get_method(const char *method_str, size_t method_len) {
@@ -716,7 +780,7 @@ ssize_t get_package_length(const Protocol *protocol, Socket *socket, PacketLengt
 uint8_t get_package_length_size(Socket *socket) {
     Connection *conn = (Connection *) socket->object;
     if (conn->websocket_status >= websocket::STATUS_HANDSHAKE) {
-        return SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_MASK_LEN + sizeof(uint64_t);
+        return SW_WEBSOCKET_MESSAGE_HEADER_SIZE;
     } else if (conn->http2_stream) {
         return SW_HTTP2_FRAME_HEADER_SIZE;
     } else {
