@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 
@@ -246,7 +246,7 @@ void Manager::start(Server *_server) {
     procctl(P_PID, 0, PROC_PDEATHSIG_CTL, &sigid);
 #endif
 
-#ifdef HAVE_PTHREAD_BARRIER
+#if defined(HAVE_PTHREAD_BARRIER) && !(defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__))
     pthread_barrier_wait(&_server->gs->manager_barrier);
 #else
     SW_START_SLEEP;
