@@ -447,7 +447,11 @@ int Server::start_master_thread() {
     if (!single_thread) {
         pthread_barrier_wait(&reactor_thread_barrier);
     }
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+    SW_START_SLEEP;
+#else
     pthread_barrier_wait(&gs->manager_barrier);
+#endif
 #else
     SW_START_SLEEP;
 #endif
@@ -728,18 +732,8 @@ Server::Server(enum Mode _mode) {
     compression_min_length = SW_COMPRESSION_MIN_LENGTH_DEFAULT;
 #endif
 
-#ifdef __linux__
-    timezone_ = timezone;
-#else
-    struct timezone tz;
-    struct timeval tv;
-    gettimeofday(&tv, &tz);
-    timezone_ = tz.tz_minuteswest * 60;
-#endif
+    timezone_ = get_timezone();
 
-    /**
-     * alloc shared memory
-     */
     gs = (ServerGS *) sw_shm_malloc(sizeof(ServerGS));
     if (gs == nullptr) {
         swoole_error("[Master] Fatal Error: failed to allocate memory for Server->gs");
@@ -788,7 +782,7 @@ int Server::create() {
     }
 
     if (enable_static_handler and locations == nullptr) {
-        locations = new std::unordered_set<std::string>;
+        locations = std::make_shared<std::unordered_set<std::string>>();
     }
 
     // Max Connections
@@ -849,8 +843,10 @@ int Server::create() {
 #ifdef HAVE_PTHREAD_BARRIER
     if (is_process_mode()) {
         pthread_barrier_init(&reactor_thread_barrier, nullptr, reactor_num + 1);
+#if !(defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__))
         pthread_barrierattr_setpshared(&gs->manager_barrier_attr, PTHREAD_PROCESS_SHARED);
         pthread_barrier_init(&gs->manager_barrier, &gs->manager_barrier_attr, 2);
+#endif
     }
 #endif
 
@@ -966,13 +962,7 @@ void Server::destroy() {
     } else {
         destroy_reactor_threads();
     }
-    if (locations) {
-        delete locations;
-    }
-    if (http_index_files) {
-        delete http_index_files;
-    }
-    for (auto i = 0; i < SW_MAX_HOOK_TYPE; i++) {
+    SW_LOOP_N(SW_MAX_HOOK_TYPE) {
         if (hooks[i]) {
             std::list<Callback> *l = reinterpret_cast<std::list<Callback> *>(hooks[i]);
             hooks[i] = nullptr;
@@ -982,8 +972,10 @@ void Server::destroy() {
 #ifdef HAVE_PTHREAD_BARRIER
     if (is_process_mode()) {
         pthread_barrier_destroy(&reactor_thread_barrier);
+#if !(defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__))
         pthread_barrier_destroy(&gs->manager_barrier);
         pthread_barrierattr_destroy(&gs->manager_barrier_attr);
+#endif
     }
 #endif
     sw_shm_free(session_list);
@@ -1235,20 +1227,7 @@ int Server::schedule_worker(int fd, SendData *data) {
     }
     // deliver tasks to idle worker processes
     else {
-        uint32_t i;
-        bool found = false;
-        for (i = 0; i < worker_num + 1; i++) {
-            key = sw_atomic_fetch_add(&worker_round_id, 1) % worker_num;
-            if (workers[key].status == SW_WORKER_IDLE) {
-                found = true;
-                break;
-            }
-        }
-        if (sw_unlikely(!found)) {
-            scheduler_warning = true;
-        }
-        swoole_trace_log(SW_TRACE_SERVER, "schedule=%d, round=%d", key, worker_round_id);
-        return key;
+        return get_idle_worker_id();
     }
 
     return key % worker_num;
@@ -1807,15 +1786,9 @@ static void Server_signal_handler(int sig) {
                            swoole_signal_to_str(WTERMSIG(status)));
         }
         break;
-        /**
-         * for test
-         */
     case SIGVTALRM:
         swoole_warning("SIGVTALRM coming");
         break;
-        /**
-         * proxy the restart signal
-         */
     case SIGUSR1:
     case SIGUSR2:
         if (serv->is_base_mode()) {
