@@ -48,6 +48,29 @@ bool StaticHandler::is_modified(const std::string &date_if_modified_since) {
     return date_format && mktime(&tm3) - (int) serv->timezone_ >= get_file_mtime();
 }
 
+bool StaticHandler::is_modified_range(const std::string &date_range) {
+    if (date_range.empty()) {
+        return false;
+    }
+
+    struct tm tm3 {};
+
+    const char *date_format = nullptr;
+
+    if (strptime(date_range.c_str(), SW_HTTP_RFC1123_DATE_GMT, &tm3) != nullptr) {
+        date_format = SW_HTTP_RFC1123_DATE_GMT;
+    } else if (strptime(date_range.c_str(), SW_HTTP_RFC1123_DATE_UTC, &tm3) != nullptr) {
+        date_format = SW_HTTP_RFC1123_DATE_UTC;
+    } else if (strptime(date_range.c_str(), SW_HTTP_RFC850_DATE, &tm3) != nullptr) {
+        date_format = SW_HTTP_RFC850_DATE;
+    } else if (strptime(date_range.c_str(), SW_HTTP_ASCTIME_DATE, &tm3) != nullptr) {
+        date_format = SW_HTTP_ASCTIME_DATE;
+    }
+    time_t file_mtime = get_file_mtime();
+    struct tm *tm_file_mtime = gmtime(&file_mtime);
+    return date_format && mktime(&tm3) != mktime(tm_file_mtime);
+}
+
 std::string StaticHandler::get_date() {
     char date_[64];
     time_t now = ::time(nullptr);
@@ -65,7 +88,7 @@ std::string StaticHandler::get_date_last_modified() {
 }
 
 bool StaticHandler::hit() {
-    char *p = task.filename;
+    char *p = filename;
     const char *url = request_url.c_str();
     size_t url_length = request_url.length();
     /**
@@ -106,15 +129,15 @@ bool StaticHandler::hit() {
     }
     dir_path = std::string(url, n);
 
-    l_filename = http_server::url_decode(task.filename, p - task.filename);
-    task.filename[l_filename] = '\0';
+    l_filename = http_server::url_decode(filename, p - filename);
+    filename[l_filename] = '\0';
 
     if (swoole_strnpos(url, n, SW_STRL("..")) == -1) {
         goto _detect_mime_type;
     }
 
     char real_path[PATH_MAX];
-    if (!realpath(task.filename, real_path)) {
+    if (!realpath(filename, real_path)) {
         if (last) {
             status_code = SW_HTTP_NOT_FOUND;
             return true;
@@ -135,7 +158,7 @@ bool StaticHandler::hit() {
 _detect_mime_type:
 // file does not exist
 check_stat:
-    if (lstat(task.filename, &file_stat) < 0) {
+    if (lstat(filename, &file_stat) < 0) {
         if (last) {
             status_code = SW_HTTP_NOT_FOUND;
             return true;
@@ -146,12 +169,12 @@ check_stat:
 
     if (S_ISLNK(file_stat.st_mode)) {
         char buf[PATH_MAX];
-        ssize_t byte = ::readlink(task.filename, buf, sizeof(buf) - 1);
+        ssize_t byte = ::readlink(filename, buf, sizeof(buf) - 1);
         if (byte <= 0) {
             return false;
         }
         buf[byte] = 0;
-        swoole_strlcpy(task.filename, buf, sizeof(task.filename));
+        swoole_strlcpy(filename, buf, sizeof(filename));
         goto check_stat;
     }
 
@@ -163,14 +186,13 @@ check_stat:
         return true;
     }
 
-    if (!swoole::mime_type::exists(task.filename) && !last) {
+    if (!swoole::mime_type::exists(filename) && !last) {
         return false;
     }
 
     if (!S_ISREG(file_stat.st_mode)) {
         return false;
     }
-    task.length = get_filesize();
 
     return true;
 }
@@ -234,7 +256,7 @@ bool StaticHandler::get_dir_files() {
         return false;
     }
 
-    DIR *dir = opendir(task.filename);
+    DIR *dir = opendir(filename);
     if (dir == nullptr) {
         return false;
     }
@@ -249,8 +271,8 @@ bool StaticHandler::get_dir_files() {
     return true;
 }
 
-bool StaticHandler::set_filename(std::string &filename) {
-    char *p = task.filename + l_filename;
+bool StaticHandler::set_filename(const std::string &filename) {
+    char *p = this->filename + l_filename;
 
     if (*p != '/') {
         *p = '/';
@@ -261,7 +283,7 @@ bool StaticHandler::set_filename(std::string &filename) {
     p += filename.length();
     *p = 0;
 
-    if (lstat(task.filename, &file_stat) < 0) {
+    if (lstat(this->filename, &file_stat) < 0) {
         return false;
     }
 
@@ -269,26 +291,179 @@ bool StaticHandler::set_filename(std::string &filename) {
         return false;
     }
 
-    task.length = get_filesize();
-
     return true;
 }
+
+void StaticHandler::parse_range(const char *range, const char *if_range) {
+    task_t _task{};
+    _task.length = 0;
+    // range
+    if (range && '\0' != *range) {
+        const char *p = range;
+        // bytes=
+        if (!SW_STRCASECT(p, strlen(range), "bytes=")) {
+            _task.offset = 0;
+            _task.length = content_length = get_filesize();
+            tasks.push_back(_task);
+            return;
+        }
+        p += 6;
+        size_t start, end, size = 0, cutoff = SIZE_MAX / 10, cutlim = SIZE_MAX % 10, suffix,
+                           _content_length = get_filesize();
+        content_length = 0;
+        for (;;) {
+            start = 0;
+            end = 0;
+            suffix = 0;
+
+            while (*p == ' ') {
+                p++;
+            }
+
+            if (*p != '-') {
+                if (*p < '0' || *p > '9') {
+                    status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                    return;
+                }
+
+                while (*p >= '0' && *p <= '9') {
+                    if (start >= cutoff && (start > cutoff || (size_t)(*p - '0') > cutlim)) {
+                        status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                        return;
+                    }
+
+                    start = start * 10 + (*p++ - '0');
+                }
+
+                while (*p == ' ') {
+                    p++;
+                }
+
+                if (*p++ != '-') {
+                    status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                    return;
+                }
+
+                while (*p == ' ') {
+                    p++;
+                }
+
+                if (*p == ',' || *p == '\0') {
+                    end = _content_length;
+                    goto found;
+                }
+
+            } else {
+                suffix = 1;
+                p++;
+            }
+
+            if (*p < '0' || *p > '9') {
+                status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                return;
+            }
+
+            while (*p >= '0' && *p <= '9') {
+                if (end >= cutoff && (end > cutoff || (size_t)(*p - '0') > cutlim)) {
+                    status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                    return;
+                }
+
+                end = end * 10 + (*p++ - '0');
+            }
+
+            while (*p == ' ') {
+                p++;
+            }
+
+            if (*p != ',' && *p != '\0' && *p != '\r') {
+                status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                return;
+            }
+
+            if (suffix) {
+                start = (end < _content_length) ? _content_length - end : 0;
+                end = _content_length - 1;
+            }
+
+            if (end >= _content_length) {
+                end = _content_length;
+
+            } else {
+                end++;
+            }
+
+        found:
+            if (start < end) {
+                if (size > SIZE_MAX - (end - start)) {
+                    status_code = SW_HTTP_RANGE_NOT_SATISFIABLE;
+                    return;
+                }
+                size += end - start;
+                _task.offset = start;
+                _task.length = end - start;
+                content_length += sw_snprintf(_task.part_header,
+                                              sizeof(_task.part_header),
+                                              "%s--%s\r\n"
+                                              "Content-Type: %s\r\n"
+                                              "Content-Range: bytes %zu-%zu/%zu\r\n\r\n",
+                                              tasks.empty() ? "" : "\r\n",
+                                              get_boundary(),
+                                              get_mimetype(),
+                                              _task.offset,
+                                              end - 1,
+                                              get_filesize()) +
+                                  _task.length;
+                tasks.push_back(_task);
+            } else if (start == 0) {
+                break;
+            }
+
+            if (*p++ != ',' || '\r' == *p || '\0' == *p) {
+                break;
+            }
+        }
+    }
+    if (_task.length > 0) {
+        if (1 == tasks.size()) {
+            content_length = _task.length;
+        } else {
+            end_part = std::string("\r\n--") + get_boundary() + "--\r\n";
+            content_length += end_part.size();
+        }
+        status_code = SW_HTTP_PARTIAL_CONTENT;
+    } else {
+        _task.offset = 0;
+        _task.length = content_length = get_filesize();
+        tasks.push_back(_task);
+    }
+    // if-range
+    if (if_range) {
+        if (is_modified_range(if_range)) {
+            tasks.clear();
+            _task.offset = 0;
+            _task.length = content_length = get_filesize();
+            tasks.push_back(_task);
+            status_code = SW_HTTP_OK;
+        }
+    }
+}
 }  // namespace http_server
 void Server::add_static_handler_location(const std::string &location) {
     if (locations == nullptr) {
-        locations = new std::unordered_set<std::string>;
+        locations = std::make_shared<std::unordered_set<std::string>>();
     }
-    locations->insert(location);
+    locations->emplace(location);
 }
 
 void Server::add_static_handler_index_files(const std::string &file) {
     if (http_index_files == nullptr) {
-        http_index_files = new std::vector<std::string>;
+        http_index_files = std::make_shared<std::vector<std::string>>();
     }
 
     auto iter = std::find(http_index_files->begin(), http_index_files->end(), file);
     if (iter == http_index_files->end()) {
-        http_index_files->push_back(file);
+        http_index_files->emplace_back(file);
     }
 }
 }  // namespace swoole
